/* * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * *
* ESP8266 ESP-01 wireless switch                                  Version 1.0 5/06/2021*
*                                                                                      *
* This code sets the ESP8266 ESP-01 up as a UDP Access Point (Server) that receives    *
* the On/Off state of a switch connected to another ESP8266 ESP-01 module that is set  *
* up as a UDP Client. The received signal drives a common Anode RGB LED where green    *
* indicates the switch is ON and red indicates the switch is OFF. The LED red & green  *
* inputs are driven by the ESP-01 GPIO0 and GPIO2 pins.                                *
* * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * */
//Libraries Included
#include <ESP8266WiFi.h>
#include <WiFiUDP.h>

//Define I/O Pins
#define     Rled        0               //GPIO0 pin assigned to red LED
#define     Gled        2               //GPIO2 pin assigned to green LED
#define     MAXSC       2               //MAXIMUM NUMBER OF CLIENTS

//Authentication Variables
char*       ssid;                       //Server WIFI Name
char*       password;                   //Server WIFI Password

//WiFi settings
IPAddress APlocal_IP(192, 168, 4, 1);   //Access Point (AP) Local IP Address
IPAddress APgateway(192, 168, 4, 1);    //Access Point (AP) Gateway Address
IPAddress APsubnet(255, 255, 255, 0);   //Access Point (AP) Subnet Address
unsigned int UDPPort = 2390;            //Local port to listen on
WiFiUDP Udp;
//Other Variables
boolean debug = false;                  //Debug flag
char packetBuffer[255]; //buffer to hold incoming packet
char* Ack1 = "BAD";
char* Ack2 = "ACK";
char* AckMsg;                           //Acknowledge message sent back to Client
String Message;                         //Incomming message from client

//====================================================================================
void setup() {
  if (debug) Serial.begin(115200);                       //Set Serial Port to 115200 baud
  pinMode(Gled, OUTPUT);                                 //Set GPIO0 as digital output to drive green LED
  pinMode(Rled, OUTPUT);                                 //Set GPIO2 as digital output to drive red LED
  digitalWrite(Gled, HIGH);                              //Turn OFF green LED
  digitalWrite(Rled, HIGH);                              //Turn OFF red LED
  //Set up a Wifi Access Point named "WirelessSwitch"
  SetWifi("WirelessSwitch","");                          //Call subroutine to start Wifi AP
}

//====================================================================================
void loop() {
  HandleClients();            //Call subroutine to handle client

}

//=================================  SUBROUTINES  ====================================
void HandleClients(){
  unsigned long CpuTime;                                 //CPU Time
  int packetSize = Udp.parsePacket();                    //Check for received data in the  packet
  if (packetSize) {                                      //Is there data in the packet?
    if (debug) {                                         //Send info to monitor if debug = true
      Serial.print("Received packet of size ");
      Serial.println(packetSize);
      Serial.print("From ");
      Serial.print(Udp.remoteIP());
      Serial.print(", port ");
      Serial.println(Udp.remotePort());
    }
    int len = Udp.read(packetBuffer, 255);               //Read the packet into packetBufffer
    if (len > 0) {
      packetBuffer[len] = 0;                             //Add a null character to end of packet
    }
    if (debug) {                                         //Send info to monitor if debug = true
      Serial.print("Contents: ");
      Serial.println(packetBuffer);
    }

    //Check received data and send appropriate acknowledge back
    Message = packetBuffer;                              //Get packet sent by Client
    AckMsg = Ack1;                                       //Preset acknowledge message to 'BAD'
    if(Message.indexOf("SW=ON") != -1) {                 //Check for ON message
      digitalWrite(Gled, LOW);                           //Turn ON green LED
      digitalWrite(Rled, HIGH);                          //Turn OFF red LED
      if (debug) Serial.println("Client reports: Switch ON"); //Send switch state to monitor
      AckMsg = Ack2;                                     //Set acknowledge message to 'ACK'
      Message = "";                                      //Clear received data
    }
    if(Message.indexOf("SW=OFF") != -1) {                //Check for OFF message
      digitalWrite(Gled, HIGH);                          //Turn OFF green LED
      digitalWrite(Rled, LOW);                           //Turn ON red LED
      if (debug) Serial.println("Client reports: Switch OFF"); //Send switch state to monitor
      AckMsg = Ack2;                                     //Set acknowledge message to 'ACK'
      Message = "";                                      //Clear received data
    }
    Udp.beginPacket(Udp.remoteIP(), Udp.remotePort());   //Send acknowledge response packet to client
    Udp.write(AckMsg);
    Udp.endPacket();
  }
}

void SetWifi(char* Name, char* Password){                //Enter with AP Name & Passwword
  WiFi.disconnect();                                     //Stop active previous WIFI

  WiFi.mode(WIFI_AP_STA);                                //Set WiFi mode to AP Station
  if (debug) Serial.println("WIFI Mode set to: AP Station");
  ssid = Name;                                           //Get SSID name passed to this subroutine
  password = Password;                                   //Get Password passed to this subroutine
  
  //Start the AccessPoint
  WiFi.softAPConfig(APlocal_IP, APgateway, APsubnet);    //SoftAPConfig (local_ip, gateway, subnet)
  WiFi.softAP(ssid, password, 1 ,0, MAXSC);              //WiFi.softAP(ssid, password, channel, hidden, max_connection)                         
  if (debug) Serial.println("WIFI Named " + String(ssid) + " Started"); //Send info to monitor if debug = true
  delay(50);                                             //Wait a bit

  IPAddress IP = WiFi.softAPIP();                        //Get server IP
  if (debug) {                                           //Send info to monitor if debug = true
    Serial.print("AccessPoint IP: ");
    Serial.println(IP);
  }
  
  Udp.begin(UDPPort);                                    //Start UDP Server
  if (debug) Serial.println("Server Started");
}
